using GDEUtils.StateMachine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum BattleActions { Move, SwitchPokemon, UseItem, Run }

public class BattleState : State<GameController>
{
    [SerializeField] BattleHUD battleHudPrefab;

    public Pokemon PlayerPokemon { get; private set; }
    public Pokemon EnemyPokemon { get; private set; }

    public int SelectedMove { get; set; }
    public BattleActions SelectedAction { get; set; }

    public StateMachine<BattleState> StateMachine { get; private set; }

    public static BattleState i { get; private set; }
    private void Awake()
    {
        i = this;
        StateMachine = new StateMachine<BattleState>(this);
    }

    public void StartState(Pokemon playerPokemon, Pokemon wildPokemon)
    {
        PlayerPokemon = playerPokemon;
        EnemyPokemon = wildPokemon;

        GameController.i.StateMachine.Push(this);
    }

    GameController gc;
    public override void Enter(GameController owner)
    {
        gc = owner;

        PlayerController.i.CanThrowPokeball = false;
        StartCoroutine(StartWildBattle());
    }

    IEnumerator StartWildBattle()
    {
        ShowBattleHUDs();
        yield return DialogueState.i.ShowDialogue($"{EnemyPokemon.Base.Name} is keeping it's guard up...");

        StateMachine.ChangeState(ActionSelectionState.i);
    }

    void ShowBattleHUDs()
    {
        if (PlayerPokemon.BattleHUD == null)
            PlayerPokemon.BattleHUD = Instantiate(battleHudPrefab, PlayerPokemon.Model.transform);

        if (EnemyPokemon.BattleHUD == null)
            EnemyPokemon.BattleHUD = Instantiate(battleHudPrefab, EnemyPokemon.Model.transform);

        PlayerPokemon.ShowHUD();
        EnemyPokemon.ShowHUD();
    }

    public override void Execute()
    {
        StateMachine.Execute();
    }

    public override void Exit()
    {
        PlayerController.i.CanThrowPokeball = true;
    }
}
